mod dirs;


use std::fs;
use std::path::PathBuf;

use anyhow::{ensure, Result};
use clap::{Parser, Subcommand};


#[derive(Debug, Parser)]
pub struct Args {
    #[command(subcommand)]
    cmd: Command,
}

#[derive(Debug, Subcommand)]
enum Command {
    /// Add a project
    Add { path: PathBuf },

    /// List existing projects
    List,
}


#[derive(Debug, Default, serde::Serialize, serde::Deserialize)]
struct Projects {
    list: Vec<PathBuf>,
}


fn main() -> Result<()> {
    let args = Args::parse();

    let mut projects = read_projects_file()?;

    let result = match &args.cmd {
        Command::Add { path } => add_project(&mut projects, path),
        Command::List => list_projects(&projects),
    };

    if let Err(err) = result {
        eprintln!("{err}");
        std::process::exit(1);
    }

    Ok(())
}


fn add_project<P>(projects: &mut Projects, path: P) -> Result<()>
where
    P: Into<PathBuf>,
{
    let path = path.into();

    ensure!(path.is_dir(), "Project path does not exists");
    ensure!(
        !projects.list.contains(&path.to_path_buf()),
        "Project path already registered"
    );

    projects.list.push(path);
    write_projects_file(projects)?;

    Ok(())
}


fn list_projects(projects: &Projects) -> Result<()> {
    for project in &projects.list {
        println!("{project:?}")
    }

    Ok(())
}

fn write_projects_file(projects: &Projects) -> Result<()> {
    let projects_file = dirs::data_path().join("projects");

    let file = fs::OpenOptions::new()
        .write(true)
        .create(true)
        .truncate(true)
        .open(projects_file)?;

    bincode::serialize_into(file, projects)?;

    Ok(())
}


fn read_projects_file() -> Result<Projects> {
    let projects_file = dirs::data_path().join("projects");

    if !projects_file.exists() {
        return Ok(Projects::default());
    }

    let file = std::fs::File::open(projects_file)?;
    let projects = bincode::deserialize_from(file)?;

    Ok(projects)
}
